@@ -739,9 +739,9 @@ def extract_scope_belleii(did, scopes):
                 return 'user.%s' % split_did[3], did
         return 'user', did
     if did.startswith('/belle/group/'):
-        if len(split_did) > 3:
-            if 'group.%s' % (split_did[3]) in scopes:
-                return 'group.%s' % split_did[3], did
+        if len(split_did) > 4:
+            if 'group.%s' % (split_did[4]) in scopes:
+                return 'group.%s' % split_did[4], did
         return 'group', did
     if did.startswith('/belle/data/') or did.startswith('/belle/Data/'):
         if len(split_did) > 4:
@@ -763,8 +763,10 @@ def extract_scope_belleii(did, scopes):
                 if split_did[4].find('fab') > -1:  # /belle/Data/proc/fab* --> data_tmp
                     return 'data_tmp', did
         return 'data_tmp', did
-    if did.startswith('/belle/ddm/functional_tests/') or did.startswith('/belle/ddm/tests/'):
+    if did.startswith('/belle/ddm/functional_tests/') or did.startswith('/belle/ddm/tests/') or did.startswith('/belle/test/ddm_test'):
         return 'test', did
+    if did.startswith('/belle/BG/'):
+        return 'data', did
     return 'other', did
 
 
